<?php

namespace Pg\Modules\Listings\Controllers;

/**
 * Listings module
 *
 * @package 	PG_RealEstate
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Listings api side controller
 *
 * @package 	PG_RealEstate
 * @subpackage 	Listings
 * @category	controllers
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Api_Listings extends \Controller
{

    /**
     * Constructor
     *
     * @return Api_Listings
     */
    public function __construct()
    {
        parent::__construct();
        $this->load->model('Listings_model');
        $this->Listings_model->set_format_settings('get_price_str', false);
    }

    /**
     * Get listing data
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function get($listing_id)
    {
        $this->load->library('pg_seo');
        if (!$listing_id) {
            log_message('error', 'listings API: Empty listing id');
            $this->set_api_content('errors', l('api_error_empty_listing_id', 'listings'));
            return false;
        }

        $format_users = array('get_contact');
        $format_listings = array('get_description', 'get_file', 'get_video', 'get_booking', 'get_views');

        $is_users_installed = $this->pg_module->is_module_installed("users");
        if ($is_users_installed) {
            $this->load->model('Users_model');
            $this->Users_model->set_format_settings(array_fill_keys($format_users, true));
        }
        $this->Listings_model->set_format_settings(array_fill_keys($format_listings, true));
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings(array_fill_keys($format_listings, false));
        if ($is_users_installed) {
            $this->Users_model->set_format_settings(array_fill_keys($format_users, false));
        }
        if (empty($listing)) {
            $this->set_api_content('messages', l('api_error_listing_not_found', 'listings'));
            return false;
        }

        if ($this->session->userdata('auth_type') == 'user') {
            $user_id = $this->session->userdata('user_id');
            $user_type = $this->session->userdata('user_type');
        } else {
            $user_id = 0;
            $user_type = '';
        }

        $data = array();

        if ($user_id == $listing['id_user']) {
            $data['is_owner'] = 1;

            $is_moderation_installed = $this->pg_module->is_module_installed("moderation");
            if ($is_moderation_installed) {
                $this->load->model('listings/models/Listings_moderation_model');

                $moderation_alerts = array();

                $admin_alert = $this->Listings_moderation_model->get_alert($listing_id);
                if (!empty($admin_alert)) {
                    $data['moderation_alerts'][$admin_alert['status']] = $admin_alert['content'];
                }

                if ($this->Listings_moderation_model->is_listing_moderated($listing_id)) {
                    $fields_for_select = $this->Listings_moderation_model->get_fields_for_moderate();
                    $this->Listings_moderation_model->set_dop_fields($fields_for_select);

                    $format_listings_moderated = array('get_description', 'get_file', 'get_video');
                    $this->Listings_model->set_format_settings(array_fill_keys($format_listings_moderated, true));
                    $moderated_data = $this->Listings_moderation_model->get_listing_by_id($listing_id);
                    $this->Listings_model->set_format_settings(array_fill_keys($format_listings_moderated, false));

                    $listing = array_merge($listing, $moderated_data);
                    $data['moderation_alerts']['info'] = l('info_your_listing_on_moderation', 'listings');
                }
            }
        } else {
            if (!$listing['status']) {
                $this->set_api_content('messages', l('api_error_listing_not_found', 'listings'));
                return false;
            }
            $data['is_owner'] = 0;
        }

        $site_url = site_url();
        $methods = array(
            'photos' => $site_url . 'api/listings/get_photos/' . $listing_id,
            'virtual_tours' => $site_url . 'api/listings/get_virtual_tours/' . $listing_id,
            'file' => $site_url . 'api/listings/get_file/' . $listing_id,
            'video' => $site_url . 'api/listings/get_video/' . $listing_id,
            'pdf' => $site_url . 'listings/view/' . $listing_id . '/overview/1',
        );

        $date_formats = $this->pg_date->get_format('date_literal', 'st');

        $data['listing_id'] = $listing_id;
        $data['logo_image'] = $listing['media']['photo']['thumbs'];
        $data['listing_is_file'] = !empty($listing['listing_file']);
        $data['listing_file'] = ($listing['is_file'] ? $listing['listing_file_content'] : '');
        $data['listing_is_video'] = !empty($listing['listing_video']);
        $data['listing_video'] = ($listing['is_video'] ? $listing['listing_video_content'] : '');
        $data['listing_video_status'] = (!empty($listing['listing_video']) ? $listing['listing_video_data']['status'] : '');
        $data['methods'] = $methods;
        $data['date_format'] = $date_formats;

        $this->load->helper('start');

        $data['listing_price_value'] = currency_format_output(array('value' => $listing['price'], 'cur_gid' => $listing['gid_currency']));
        if ($listing['price_reduced']) {
            $data['listing_price_reduced_value'] = currency_format_output(array('value' => $listing['price_reduced'], 'cur_gid' => $listing['gid_currency']));
        } else {
            $data['listing_price_reduced_value'] = '';
        }

        // photo settings
        $this->load->model('listings/models/Listings_uploads_model');
        $gallery_type_config = $this->Listings_uploads_model->get_gallery_type();
        $gallery_type_config['max'] = $gallery_type_config['max_items_count'];

        $upload_config_gid = $gallery_type_config['gid_upload_config'];

        $this->load->model('Uploads_model');
        $gallery_type_config['upload_settings'] = $this->Uploads_model->get_config($upload_config_gid);

        $max_size = $gallery_type_config['upload_settings']['max_size'];

        $i = 1;
        while ($max_size > 1000) {
            $max_size /= 1024;
            $i++;
        }
        $max_size = round($max_size, 1);
        if ($max_size) {
            $gallery_type_config['upload_settings']['max_size_str'] = $max_size . ' ' . l('text_filesize_' . $i, 'listings');
        }

        $data['listing_photo_settings'] = $gallery_type_config;

        // video settings
        $video_type_config = $this->Listings_uploads_model->get_video_type();
        $video_type_config['max'] = 1;

        $max_size = $video_type_config['max_size'];

        $i = 1;
        while ($max_size > 1000) {
            $max_size /= 1024;
            $i++;
        }
        $max_size = round($max_size, 1);
        if ($max_size)
            $video_type_config['max_size_str'] = $max_size . ' ' . l('text_filesize_' . $i, 'listings');
        $data['listing_video_settings'] = $video_type_config;

        $this->load->helper('seo');
        $data['seolink'] = urldecode(rewrite_link('listings', 'view', $listing));

        $this->load->helper('date_format');

        $data['listing_user']['logo_image'] = $listing['user']['media']['user_logo']['thumbs'];
        $data['listing_date_activity_output'] = tpl_date_format($listing['date_activity'], $date_formats);
        $data['listing_date_available_output'] = tpl_date_format($listing['date_available'], $date_formats);
        $data['listing_date_activity_output'] = tpl_date_format($listing['date_activity'], $date_formats);
        $data['listing_date_created_output'] = tpl_date_format($listing['date_created'], $date_formats);
        $data['listing_date_expire_output'] = tpl_date_format($listing['date_expire'], $date_formats);
        $data['listing_date_modified_output'] = tpl_date_format($listing['date_modified'], $date_formats);
        $data['listing_date_open_output'] = tpl_date_format($listing['date_open'], $date_formats);
        $data['listing_featured_date_end_output'] = tpl_date_format($listing['featured_date_end'], $date_formats);
        $data['listing_highlight_date_end_output'] = tpl_date_format($listing['highlight_date_end'], $date_formats);
        $data['listing_lift_up_city_date_end_output'] = tpl_date_format($listing['lift_up_city_date_end'], $date_formats);
        $data['listing_lift_up_country_date_end_output'] = tpl_date_format($listing['lift_up_country_date_end'], $date_formats);
        $data['listing_lift_up_date_end_output'] = tpl_date_format($listing['lift_up_date_end'], $date_formats);
        $data['listing_lift_up_region_date_end_output'] = tpl_date_format($listing['lift_up_region_date_end'], $date_formats);
        $data['listing_slide_show_date_end_output'] = tpl_date_format($listing['slide_show_date_end'], $date_formats);

        if ($this->pg_module->is_module_installed('listings_booking')) {
            foreach ($listing['booking']['periods'] as $i => $period) {
                $period['price_str'] = str_replace('&nbsp;', ' - ', strip_tags(currency_format_output(array('value' => $period['price'], 'gid' => $listing['gid_currency']))));
                $listing['booking']['periods'][$i] = $period;
            }
        }

        if ($data['is_owner']) {
            if ($this->pg_module->is_module_installed('users_services')) {
                // check pay services exists
                $this->load->model('Services_model');
                // check pay services exists
                $templates_gids = array(
                    'listings_featured_template',
                    'listings_lift_up_template',
                    'listings_lift_up_country_template',
                    'listings_lift_up_region_template',
                    'listings_lift_up_city_template',
                    'listings_highlight_template',
                    'listings_slide_show_template',
                );
                $services = $this->Services_model->get_user_type_active_services($user_type, $templates_gids);

                if (count($services) > 0) {
                    $data['use_services'] = 1;
                } else {
                    $params = array(
                        'where' => array(
                            'id_user' => $user_id,
                            'status' => 1,
                        ),
                        'where_in' => array(
                            'template_gid' => $templates_gids,
                        ),
                    );

                    $this->load->model('listings/models/Listings_service_model');
                    $user_services = $this->Listings_service_model->get_services_list(null, $params);

                    $data['use_services'] = count($user_services) > 0 ? 1 : 0;
                }

                $post_templates_gids = array(
                    'post_template',
                    'combined_template'
                );
                $post_services = $this->Services_model->get_user_type_active_services($user_type, $post_templates_gids);
                $data['use_post_services'] = count($post_services) > 0;
            } else {
                $data['use_services'] = 0;
                $data['use_post_services'] = 0;
            }

            if ($listing['status'] == 0 && strtotime($listing['date_expire']) > time()) {
                $data['can_reactivate'] = 1;
            }

            if (strtotime($listing['date_expire'])) {
                $data['listing_date_expire_str'] = tpl_date_format($listing['date_expire'], $date_formats);
            }

            if ($data['use_post_services']) {
                // get user services
                $params = array(
                    'where_in' => array(
                        'template_gid' => array('post_template', 'combined_template',),
                    ),
                    'where' => array(
                        'id_user' => $user_id,
                        'status' => '1',
                    ),
                );
                $this->load->model('Users_services_model');
                $data['user_post_services'] = $this->Users_services_model->get_services_list(null, $params);
            } else {
                $data['default_activation_period'] = $this->pg_module->get_module_config('listings', 'default_activation_period');
                $data['end_activation_date'] = date('Y-m-d H:i:s', time() + $data['default_activation_period'] * 24 * 60 * 60);
                $data['end_activation_date_str'] = tpl_date_format($data['end_activation_date'], $date_formats);
            }
        } else {
            $this->load->model('listings/models/Listings_visitor_model');
            $this->Listings_visitor_model->check_views_count($user_id, $listing_id);
        }

        unset($listing['id']);
        unset($listing['logo_image']);
        unset($listing['media']);
        unset($listing['listing_file']);
        unset($listing['listing_file_date']);
        unset($listing['is_file']);
        unset($listing['listing_video']);
        unset($listing['listing_video_image']);
        unset($listing['listing_video_data']);
        unset($listing['is_video']);

        foreach ($listing as $name => $value) {
            $data['listing_' . $name] = $value;
        }

        $this->set_api_content('data', $data);
    }

    /**
     * Get listing photos
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function get_photos($listing_id)
    {
        if (!$listing_id) {
            log_message('error', 'listings API: Empty listing id');
            $this->set_api_content('errors', l('api_error_empty_listing_id', 'listings'));
            return false;
        }

        $format_settings = array(
            'get_user' => false,
            'get_location' => false,
            'get_photos' => true,
        );
        $this->Listings_model->set_format_settings($format_settings);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $format_settings = array(
            'get_user' => true,
            'get_location' => true,
            'get_photos' => false,
        );
        $this->Listings_model->set_format_settings($format_settings);

        if (empty($listing)) {
            $this->set_api_content('messages', l('api_error_listing_not_found', 'listings'));
            return false;
        }

        $data = array('photos' => array());

        if (empty($listing['photos'])) {
            $data['photo_default'] = array(
                'source' => $listing['photo_default']['media']['url'],
                'big' => $listing['photo_default']['media']['thumbs']['620_400'],
                'middle' => $listing['photo_default']['media']['thumbs']['200_200'],
                'small' => $listing['photo_default']['media']['thumbs']['60_60'],
            );
        } else {
            foreach ($listing['photos'] as $photo) {
                if (!$photo['status'])
                    continue;
                $data['photos'][] = array(
                    'id' => $photo['id'],
                    'source' => $photo['media']['url'],
                    'big' => $photo['media']['thumbs']['620_400'],
                    'middle' => $photo['media']['thumbs']['200_200'],
                    'small' => $photo['media']['thumbs']['60_60'],
                    'comment' => $photo['comment'],
                );
            }
        }

        $this->set_api_content('data', $data);
    }

    /**
     * Get listing virtual tours
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function get_virtual_tours($listing_id)
    {
        if (!$listing_id) {
            log_message('error', 'listings API: Empty listing id');
            $this->set_api_content('errors', l('api_error_empty_listing_id', 'listings'));
            return false;
        }

        $format_settings = array(
            'get_user' => false,
            'get_location' => false,
            'get_virtual_tours' => true,
        );
        $this->Listings_model->set_format_settings($format_settings);

        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        if (empty($listing)) {
            $this->set_api_content('messages', l('api_error_listing_not_found', 'listings'));
            return false;
        }

        $format_settings = array(
            'get_user' => true,
            'get_location' => true,
            'get_virtual_tours' => false,
        );
        $this->Listings_model->set_format_settings($format_settings);

        $data = array('virtual_tours' => array(), 'max_panorama' => $listing['max_panorama']);
        foreach ($listing['virtual_tour'] as $panorama) {
            if (!$panorama['status'])
                continue;
            $data['virtual_tours'][] = array(
                'source' => $panorama['media']['url'],
                'big' => $panorama['media']['thumbs']['big'],
                'middle' => $panorama['media']['thumbs']['middle'],
                'small' => $panorama['media']['thumbs']['small'],
                'comment' => $panorama['comment'],
            );
        }
        $this->set_api_content('data', $data);
    }

    /**
     * Set listing data
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function set($listing_id = null)
    {
        $auth_type = $this->session->userdata('auth_type');
        if ('user' != $auth_type) {
            log_message('error', 'listings API: Wrong auth type ("' . $auth_type . '")');
            $this->set_api_content('errors', l('api_error_wrong_auth_type', 'listings'));
            return false;
        }

        $user_id = $this->session->userdata('user_id');

        $post_data = $this->input->post('data', true);

        if ($listing_id) {
            $this->Listings_model->set_format_settings('use_format', false);
            $data = $this->Listings_model->get_listing_by_id($listing_id);
            $this->Listings_model->set_format_settings('use_format', true);

            if (!$data || $user_id != $data['id_user']) {
                log_message('error', 'listings API: Invalid listing id');
                $this->set_api_content('errors', l('api_error_invalid_listing_id', 'listings'));
                return false;
            }

            if (isset($post_data['category'])) {
                $pos = strpos($post_data['category'], '_');
                $post_data['property_type'] = substr($post_data['category'], $pos + 1);
                unset($post_data['category']);
            }
        } else {
            $data = array();

            $post_data['id_user'] = $user_id;

            if (isset($post_data['type'])) {
                $post_data['id_type'] = $this->Listings_model->get_operation_type_by_gid($post_data['type']);
            }

            if (isset($post_data['category'])) {
                $pos = strpos($post_data['category'], '_');
                $post_data['id_category'] = substr($post_data['category'], 0, $pos);
                $post_data['property_type'] = substr($post_data['category'], $pos + 1);
                unset($post_data['category']);
            }
        }

        $property_type_gid = $this->Listings_model->get_field_editor_type($data);
        if ($property_type_gid) {
            $this->load->model('listings/models/Listings_field_editor_model');
            $fields_for_select = $this->Listings_field_editor_model->get_fields_for_select($property_type_gid);
            foreach ($fields_for_select as $field) {
                $post_data[$field] = $this->input->post($field, true);
            }
        }

        if (empty($post_data['lat']) || empty($post_data['lon'])) {
            $this->Listings_model->set_format_settings(array('get_location' => false));
        }

        $validate_data = $this->Listings_model->validate_listing($listing_id, $post_data);
        if (!empty($validate_data['errors'])) {
            $this->set_api_content('errors', $validate_data['errors']);
            return false;
        } elseif (empty($validate_data['data'])) {
            $this->set_api_content('errors', l('error_empty_listing_data', 'listings'));
        } else {
            if ($listing_id) {
                $this->Listings_model->save_listing($listing_id, $validate_data['data'], true);
                $this->set_api_content('messages', l('success_update_listing', 'listings'));
            } else {
                $listing_id = $this->Listings_model->save_listing(null, $validate_data['data'], true);
                $this->set_api_content('messages', l('success_add_listing', 'listings'));
            }
        }

        $data['listing_id'] = $listing_id;

        $this->set_api_content('data', $data);
    }

    /**
     * Create listing
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function create()
    {
        $this->set(0);
    }

    /**
     * Remove listing
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function delete($listing_id)
    {
        $auth_type = $this->session->userdata('auth_type');
        if ('user' != $auth_type) {
            log_message('error', 'listings API: Wrong auth type ("' . $auth_type . '")');
            $this->set_api_content('errors', l('api_error_wrong_auth_type', 'listings'));
            return false;
        }

        $user_id = $this->session->userdata('user_id');

        $this->Listings_model->set_format_settings('use_format', false);
        $data = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);

        if (!$data) {
            log_message('error', 'listings API: Empty listing id');
            $this->set_api_content('errors', l('api_error_empty_listing_id', 'listings'));
            return false;
        }

        if ($user_id != $data['id_user']) {
            log_message('error', 'listings API: Invalid listing id');
            $this->set_api_content('errors', l('api_error_invalid_listing_id', 'listings'));
            return false;
        }

        $this->Listings_model->delete_listing($listing_id);
        $this->set_api_content('messages', l('success_listings_delete', 'listings'));
    }

    /**
     * Upload photo to listing gallery
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function photo_upload($listing_id)
    {
        $auth_type = $this->session->userdata('auth_type');
        if ('user' != $auth_type) {
            log_message('error', 'listings API: Wrong auth type ("' . $auth_type . '")');
            $this->set_api_content('errors', l('api_error_wrong_auth_type', 'listings'));
            return false;
        }

        $user_id = $this->session->userdata('user_id');

        $listing_id = intval($listing_id);
        if (!$listing_id) {
            log_message('error', 'listings API: Empty listing id');
            $this->set_api_content('errors', l('api_error_empty_listing_id', 'listings'));
            return false;
        }

        $this->Listings_model->set_format_settings('use_format', false);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);
        if (!$listing || $user_id != $listing['id_user']) {
            log_message('error', 'listings API: Invalid listing id');
            $this->set_api_content('errors', l('api_error_invalid_listing_id', 'listings'));
            return false;
        }

        $photo_id = intval($this->input->post('photo_id', true));
        if ($photo_id) {
            $this->load->model('Upload_gallery_model');
            $photo = $this->Upload_gallery_model->get_file_by_id($photo_id);
            if ($listing_id != $photo['object_id']) {
                log_message('error', 'listings API: Invalid listing id');
                $this->set_api_content('errors', l('api_error_invalid_listing_id', 'listings'));
                return false;
            }
        }

        $this->load->model('listings/models/Listings_uploads_model');
        $return = $this->Listings_uploads_model->save_photo($listing_id, $photo_id, 'photo_file');
        if (!empty($return['errors'])) {
            $this->set_api_content('errors', implode('<br>', $return['errors']));
        } else {
            $this->set_api_content('messages', l('success_photo_uploaded', 'listings'));
        }
    }

    /**
     * Remove photo from listing gallery
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function photo_delete($listing_id)
    {
        if ($this->session->userdata('auth_type') != 'user') {
            $this->set_api_content('error', l('error_not_access', 'listings'));
            exit;
        }

        $listing_id = intval($listing_id);
        if (!$listing_id) {
            $this->set_api_content('error', l('error_not_access', 'listings'));
            exit;
        }

        $photo_id = intval($this->input->post('photo_id', true));
        if (!$photo_id) {
            $this->set_api_content('error', l('error_not_access', 'listings'));
            exit;
        }

        $this->Listings_model->set_format_settings('get_photos_all', true);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('get_photos_all', false);

        $user_id = $this->session->userdata('user_id');
        if ($user_id != $listing['id_user']) {
            $this->set_api_content('error', l('error_not_access', 'listings'));
            exit;
        }

        // remove photo
        $this->load->model('listings/models/Listings_uploads_model');
        $result = $this->Listings_uploads_model->delete_photo($listing_id, $photo_id);
        if (!$result) {
            $this->set_api_content('error', l('error_not_access', 'listings'));
            exit;
        }

        $this->set_api_content('messages', l('photo_successfully_deleted', 'listings'));
    }

    /**
     * Upload panorama to listing gallery
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function panorama_upload($listing_id)
    {
        $auth_type = $this->session->userdata('auth_type');
        if ('user' != $auth_type) {
            log_message('error', 'listings API: Wrong auth type ("' . $auth_type . '")');
            $this->set_api_content('errors', l('api_error_wrong_auth_type', 'listings'));
            return false;
        }

        $user_id = $this->session->userdata('user_id');

        $listing_id = intval($listing_id);
        if (!$listing_id) {
            log_message('error', 'listings API: Empty listing id');
            $this->set_api_content('errors', l('api_error_empty_listing_id', 'listings'));
            return false;
        }

        $this->Listings_model->set_format_settings('use_format', false);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);
        if (!$listing || $user_id != $listing['id_user']) {
            log_message('error', 'listings API: Invalid listing id');
            $this->set_api_content('errors', l('api_error_invalid_listing_id', 'listings'));
            return false;
        }

        $panorama_id = intval($this->input->post('panorama_id', true));

        $this->load->model('Upload_gallery_model');
        if ($panorama_id) {
            $panorama = $this->Upload_gallery_model->get_file_by_id($panorama_id);
            if ($listing_id != $photo['object_id']) {
                log_message('error', 'listings API: Invalid listing id');
                $this->set_api_content('errors', l('api_error_invalid_listing_id', 'listings'));
                return false;
            }
        }

        $this->load->model('listings/models/Listings_uploads_model');
        $return = $this->Listings_uploads_model->save_panorama($listing_id, $panorama_id, 'panorama_file');
        if (!empty($return['errors'])) {
            $this->set_api_content('errors', implode('<br>', $return['errors']));
        } else {
            $this->set_api_content('messages', l('success_panorama_uploaded', 'listings'));
        }
    }

    /**
     * Remove panorama from listing gallery
     * 
     * @param integer $listing_id listing identifier
     * @param integer $panorama_id panorama identifier
     * @return void
     */
    public function panorama_delete($listing_id, $panorama_id)
    {
        $auth_type = $this->session->userdata('auth_type');
        if ('user' != $auth_type) {
            log_message('error', 'listings API: Wrong auth type ("' . $auth_type . '")');
            $this->set_api_content('errors', l('api_error_wrong_auth_type', 'listings'));
            return false;
        }

        $user_id = $this->session->userdata('user_id');

        $listing_id = intval($listing_id);
        if (!$listing_id) {
            log_message('error', 'listings API: Empty listing id');
            $this->set_api_content('errors', l('api_error_empty_listing_id', 'listings'));
            return false;
        }

        $panorama_id = intval($panorama_id);
        if (!$panorama_id) {
            log_message('error', 'listings API: Empty panorama id');
            $this->set_api_content('errors', l('api_error_empty_panorama_id', 'listings'));
            return false;
        }

        $this->Listings_model->set_format_settings('use_format', false);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);
        if (!$listing || $user_id != $listing['id_user']) {
            log_message('error', 'listings API: Invalid listing id');
            $this->set_api_content('errors', l('api_error_invalid_listing_id', 'listings'));
            return false;
        }

        // remove panorama
        $this->load->model('listings/models/Listings_uploads_model');
        $result = $this->Listings_uploads_model->delete_photo($listing_id, $panorama_id);
        if (!$result) {
            $this->set_api_content('errors', l('error_panorama_deleted', 'listings'));
        } else {
            $this->set_api_content('messages', l('success_panorama_deleted', 'listings'));
        }
    }

    /**
     * Upload file to listing
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function file_upload($listing_id)
    {
        $auth_type = $this->session->userdata('auth_type');
        if ('user' != $auth_type) {
            log_message('error', 'listings API: Wrong auth type ("' . $auth_type . '")');
            $this->set_api_content('errors', l('api_error_wrong_auth_type', 'listings'));
            return false;
        }

        $user_id = $this->session->userdata('user_id');

        $listing_id = intval($listing_id);
        if (!$listing_id) {
            log_message('error', 'listings API: Empty listing id');
            $this->set_api_content('errors', l('api_error_empty_listing_id', 'listings'));
            return false;
        }

        $this->Listings_model->set_format_settings('use_format', false);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);
        if (!$listing || $user_id != $listing['id_user']) {
            log_message('error', 'listings API: Invalid listing id');
            $this->set_api_content('errors', l('api_error_invalid_listing_id', 'listings'));
            return false;
        }

        $this->load->model('listings/models/Listings_uploads_model');
        $post_data = $this->Listings_uploads_model->save_file($listing_id, 'listing_file', true);

        ///// delete listing file
        if ($listing['listing_file']) {
            $this->load->model('File_uploads_model');
            $this->File_uploads_model->delete_upload($this->Listings_uploads_model->file_config_id, $listing['prefix'], $listing['listing_file']);
            if (!isset($post_data['data']['file']))
                $post_data['data']['listing_file'] = '';
        }

        $data['listing_file'] = $post_data['data']['file'];
        $this->Listings_model->save_listing($listing_id, $data);

        $this->set_api_content('messages', l('success_file_uploaded', 'listings'));
    }

    /**
     * Remove file from listing
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function file_delete($listing_id)
    {
        $auth_type = $this->session->userdata('auth_type');
        if ('user' != $auth_type) {
            log_message('error', 'listings API: Wrong auth type ("' . $auth_type . '")');
            $this->set_api_content('errors', l('api_error_wrong_auth_type', 'listings'));
            return false;
        }

        $user_id = $this->session->userdata('user_id');

        $listing_id = intval($listing_id);
        if (!$listing_id) {
            log_message('error', 'listings API: Empty listing id');
            $this->set_api_content('errors', l('api_error_empty_listing_id', 'listings'));
            return false;
        }

        $this->Listings_model->set_format_settings('use_format', false);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);
        if (!$listing || $user_id != $listing['id_user']) {
            log_message('error', 'listings API: Invalid listing id');
            $this->set_api_content('errors', l('api_error_invalid_listing_id', 'listings'));
            return false;
        }

        if (!$listing['listing_file']) {
            log_message('error', 'listings API: Empty listing file');
            $this->set_api_content('errors', l('api_error_empty_file', 'listings'));
            return false;
        }

        $this->load->model('File_uploads_model');
        $this->load->model('listings/models/Listings_uploads_model');
        $this->File_uploads_model->delete_upload($this->Listings_uploads_model->file_config_id, $listing['prefix'], $listing['listing_file']);

        $data['listing_file'] = '';
        $this->Listings_model->save_listing($listing_id, $data);

        $this->set_api_content('messages', l('success_file_deleted', 'listings'));
    }

    /**
     * Upload video to listing
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function video_upload($listing_id)
    {
        $auth_type = $this->session->userdata('auth_type');
        if ('user' != $auth_type) {
            log_message('error', 'listings API: Wrong auth type ("' . $auth_type . '")');
            $this->set_api_content('errors', l('api_error_wrong_auth_type', 'listings'));
            return false;
        }

        $user_id = $this->session->userdata('user_id');

        $listing_id = intval($listing_id);
        if (!$listing_id) {
            log_message('error', 'listings API: Empty listing id');
            $this->set_api_content('errors', l('api_error_empty_listing_id', 'listings'));
            return false;
        }

        $this->Listings_model->set_format_settings('use_format', false);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);
        if (!$listing || $user_id != $listing['id_user']) {
            log_message('error', 'listings API: Invalid listing id');
            $this->set_api_content('errors', l('api_error_invalid_listing_id', 'listings'));
            return false;
        }

        $this->load->model('listings/models/Listings_uploads_model');
        $return = $this->Listings_uploads_model->save_video($listing_id, 'listing_video', '', true);
        if (!empty($return['errors']) && !$return['success']) {
            $this->set_api_content('errors', $return['errors']);
        } else {
            $this->set_api_content('messages', l('success_video_uploaded', 'listings'));
        }
    }

    /**
     * Remove video from listing
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function video_delete($listing_id)
    {
        $auth_type = $this->session->userdata('auth_type');
        if ('user' != $auth_type) {
            log_message('error', 'listings API: Wrong auth type ("' . $auth_type . '")');
            $this->set_api_content('errors', l('api_error_wrong_auth_type', 'listings'));
            return false;
        }

        $user_id = $this->session->userdata('user_id');

        $listing_id = intval($listing_id);
        if (!$listing_id) {
            log_message('error', 'listings API: Empty listing id');
            $this->set_api_content('errors', l('api_error_empty_listing_id', 'listings'));
            return false;
        }

        $this->Listings_model->set_format_settings(array('get_user' => false, 'get_location' => false));
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings(array('get_user' => true, 'get_location' => true));
        if (!$listing || $user_id != $listing['id_user']) {
            log_message('error', 'listings API: Invalid listing id');
            $this->set_api_content('errors', l('api_error_invalid_listing_id', 'listings'));
            return false;
        }

        if (!$listing['listing_video']) {
            log_message('error', 'listings API: Empty listing video');
            $this->set_api_content('errors', l('api_error_empty_video', 'listings'));
            return false;
        }

        $this->load->model('listings/models/Listings_uploads_model');
        $video_config_id = $this->Listings_uploads_model->video_config_id;

        $this->load->model('Video_uploads_model');
        $this->Video_uploads_model->delete_upload($video_config_id, $listing['prefix'], $listing['listing_video'], $listing['listing_video_image'], $listing['listing_video_data']['data']['upload_type']);

        $data['listing_video'] = '';
        $data['listing_video_image'] = '';
        $data['listing_video_data'] = '';
        $this->Listings_model->save_listing($listing_id, $data);

        $this->set_api_content('messages', l('success_video_deleted', 'listings'));
    }

    /**
     * Get my listings
     *  
     * @param integer $page page of results
     * @param string $order sorting order 
     * @param string $order_direction order direction
     * @return void
     */
    public function my($page = 1, $order = 'date_modified', $order_direction = 'DESC')
    {
        $auth_type = $this->session->userdata('auth_type');
        if ('user' != $auth_type) {
            log_message('error', 'listings API: Wrong auth type ("' . $auth_type . '")');
            $this->set_api_content('errors', l('api_error_wrong_auth_type', 'listings'));
            return false;
        }

        $user_id = $this->session->userdata('user_id');
        $user_type = $this->session->userdata('user_type');

        $criteria = $this->input->post('criteria');

        if (!empty($criteria['type'])) {
            $data['operation_type'] = $criteria['type'];
            unset($criteria['type']);
        }

        //if(empty($data['type'])) $data['operation_type'] = $this->Listings_model->get_operation_type_gid_default();

        $data['order'] = $order;
        if (!$data['order'])
            $data['order'] = 'date_modified';

        $data['order_direction'] = $order_direction;
        if (!$data['order_direction'])
            $data['order_direction'] = 'DESC';

        $data['page'] = $page;
        if (empty($data['page']))
            $data['page'] = 1;

        $items_on_page = $this->pg_module->get_module_config('start', 'index_items_per_page');

        $filters = array('user' => $user_id);

        $data['listings'] = array();

        $data['listings_count'] = $this->Listings_model->get_listings_count($filters);
        if ($data['listings_count'] > 0) {
            $order_array = array();
            if ($data['order'] == 'price') {
                $order_array['price_sorting'] = $data['order_direction'];
            } else {
                $order_array[$data['order']] = $data['order_direction'];
            }
            $this->Listings_model->set_format_settings(array('get_user' => false, 'get_moderation' => true));
            $listings = $this->Listings_model->get_listings_list($filters, $page, $items_on_page, $order_array);
            $this->Listings_model->set_format_settings(array('get_user' => true, 'get_moderation' => false));

            foreach ($listings as $i => $listing) {
                $data['listings'][$i] = array(
                    'listing_id' => $listing['id'],
                    'logo_image' => $listing['media']['photo']['thumbs'],
                );

                unset($listing['id']);
                unset($listing['logo_image']);
                unset($listing['media']);
                unset($listing['listing_file']);
                unset($listing['listing_file_date']);
                unset($listing['listing_video']);
                unset($listing['listing_video_image']);
                unset($listing['listing_video_data']);

                foreach ($listing as $name => $value) {
                    $data['listings'][$i]['listing_' . $name] = $value;
                }
            }
        } else {
            $this->set_api_content('messages', l('api_error_listing_not_found', 'listings'));
        }

        $data['date_format'] = $this->pg_date->get_format('date_literal', 'st');
        $data['date_time_format'] = $this->pg_date->get_format('date_time_literal', 'st');

        if ($this->pg_module->is_module_installed('services')) {
            $this->load->model('Services_model');
            $data['featured_service_active'] = $this->Services_model->is_service_active('listings_featured_service');
            $data['highlight_service_active'] = $this->Services_model->is_service_active('listings_highlight_service');
            $data['lift_up_service_active'] = $this->Services_model->is_service_active('listings_lift_up_service');
            $data['lift_up_country_service_active'] = $this->Services_model->is_service_active('listings_lift_up_country_service');
            $data['lift_up_region_service_active'] = $this->Services_model->is_service_active('listings_lift_up_region_service');
            $data['lift_up_city_service_active'] = $this->Services_model->is_service_active('listings_lift_up_city_service');
            $data['slide_show_service_active'] = $this->Services_model->is_service_active('listings_slide_show_service');
        }

        $this->set_api_content('data', $data);
    }

    /**
     * Search listings by filters
     *
     * @param integer $page page of results
     * @param string $order sorting order
     * @param string $order_direction order direction asc/desc
     * @return void
     */
    public function search($page = 1, $order = 'default', $order_direction = 'DESC')
    {
        $filters = array();

        $data = (array) $this->input->post('criteria', true);

        // category
        if (isset($data['category'])) {
            $category_chunks = explode('_', $data['category']);
            $filters['id_category'] = $category_chunks[0];
            if (count($category_chunks) > 1 && $category_chunks[1]) {
                $filters['property_type'] = $category_chunks[1];
            }
            unset($data['category']);
        }

        foreach ($data as $key => $value) {
            if (!empty($value)) {
                $filters[$key] = $value;
            }
        }

        $filters['active'] = '1';

        $data['order'] = trim(strip_tags($order));
        if (!$data['order'])
            $data['order'] = 'date_created';

        $data['order_direction'] = strtoupper(trim(strip_tags($order_direction)));
        if ($data['order_direction'] != 'DESC')
            $data['order_direction'] = 'ASC';

        $data['page'] = $page;
        if (!$data['page'])
            $data['page'] = 1;

        $items_on_page = $this->pg_module->get_module_config('start', 'index_items_per_page');

        $data['listings'] = array();
        $data['listings_count'] = $this->Listings_model->get_listings_count($filters);

        $use_leader = false;
        if ($data['listings_count'] > 0) {
            $order_array = array();

            if ($order == 'default') {
                if (!empty($filters['id_city']))
                    $order_array['is_lift_up_city'] = 'DESC';
                if (!empty($filters['id_region']))
                    $order_array['is_lift_up_region'] = 'DESC';
                if (!empty($filters['id_country']))
                    $order_array['is_lift_up_country'] = 'DESC';
                $order_array['is_lift_up'] = 'DESC';
                $order_array['id'] = 'DESC';
                $use_leader = true;
            }else {
                $order_array[$data['order']] = $data['order_direction'];
            }

            $this->Listings_model->set_format_settings('get_user', false);
            $listings = $this->Listings_model->get_listings_list($filters, $data['page'], $items_on_page, $order_array);
            $this->Listings_model->set_format_settings('get_user', true);

            foreach ($listings as $i => $listing) {
                $data['listings'][$i] = array(
                    'listing_id' => $listing['id'],
                    'logo_image' => $listing['media']['photo']['thumbs'],
                );

                unset($listing['id']);
                unset($listing['logo_image']);
                unset($listing['media']);
                unset($listing['listing_file']);
                unset($listing['listing_file_date']);
                unset($listing['listing_video']);
                unset($listing['listing_video_image']);
                unset($listing['listing_video_data']);

                foreach ($listing as $name => $value) {
                    $data['listings'][$i]['listing_' . $name] = $value;
                }
            }
        } else {
            $this->set_api_content('messages', l('api_error_listing_not_found', 'listings'));
        }

        $data['date_format'] = $this->pg_date->get_format('date_literal', 'st');
        $data['date_time_format'] = $this->pg_date->get_format('date_time_literal', 'st');

        $use_save_search = $this->session->userdata('auth_type') == 'user';
        $data['use_save_search'] = $use_save_search;

        $data['use_leader'] = $use_leader;

        $this->set_api_content('data', $data);
    }

    /**
     * Get saved listings
     *
     * @param integer $page page of results
     * @param string $order sorting order
     * @param string $order_direction order direction asc/desc
     * @return void
     */
    public function saved($page = 1, $order = 'date_created', $order_direction = 'DESC')
    {

        $auth_type = $this->session->userdata('auth_type');
        if ('user' != $auth_type) {
            log_message('error', 'listings API: Wrong auth type ("' . $auth_type . '")');
            $this->set_api_content('errors', l('api_error_wrong_auth_type', 'listings'));
            return false;
        }

        $user_id = $this->session->userdata('user_id');

        $filters = array('ids' => array(), 'active' => 1);

        $this->load->model('listings/models/Saved_listings_model');
        $saved_listings = $this->Saved_listings_model->get_saved_list_by_user($user_id);
        foreach ($saved_listings as $saved) {
            $filters['ids'][] = $saved['id_listing'];
        }

        if (empty($filters['ids'])) {
            $this->set_api_content('messages', l('api_error_listing_not_found', 'listings'));
            return false;
        }

        $data['order'] = trim(strip_tags($order));
        if (!$data['order'])
            $data['order'] = 'date_created';

        $data['order_direction'] = strtoupper(trim(strip_tags($data['order_direction'])));
        if ($data['order_direction'] != 'DESC')
            $data['order_direction'] = 'ASC';

        $data['page'] = $page;
        if (!$data['page'])
            $data['page'] = 1;

        $items_on_page = $this->pg_module->get_module_config('start', 'index_items_per_page');

        $data['listings'] = array();
        $data['listings_count'] = $this->Listings_model->get_listings_count($filters);

        $use_leader = false;
        if ($data['listings_count'] > 0) {
            $order_array = array();

            if ($order == 'default') {
                if (isset($data['id_city']) && !empty($data['id_city']))
                    $order_array['is_lift_up_city'] = 'DESC';
                if (isset($data['id_region']) && !empty($data['id_region']))
                    $order_array['is_lift_up_region'] = 'DESC';
                if (isset($data['id_country']) && !empty($data['id_country']))
                    $order_array['is_lift_up_country'] = 'DESC';
                $order_array['is_lift_up_date'] = 'DESC';
                $order_array['id'] = 'DESC';
                $use_leader = true;
            }else {
                $order_array[$data['order']] = $data['order_direction'];
            }

            $this->Listings_model->set_format_settings('get_user', false);
            $listings = $this->Listings_model->get_listings_list($filters, $data['page'], $items_on_page, $order_array);
            $this->Listings_model->set_format_settings('get_user', true);

            foreach ($listings as $i => $listing) {
                $data['listings'][$i] = array(
                    'listing_id' => $listing['id'],
                    'logo_image' => $listing['media']['photo']['thumbs'],
                );

                unset($listing['id']);
                unset($listing['logo_image']);
                unset($listing['media']);
                unset($listing['listing_file']);
                unset($listing['listing_file_date']);
                unset($listing['listing_video']);
                unset($listing['listing_video_image']);
                unset($listing['listing_video_data']);

                foreach ($listing as $name => $value) {
                    $data['listings'][$i]['listing_' . $name] = $value;
                }
            }
        } else {
            $this->set_api_content('messages', l('api_error_listing_not_found', 'listings'));
        }

        $data['date_format'] = $this->pg_date->get_format('date_literal', 'st');
        $data['date_time_format'] = $this->pg_date->get_format('date_time_literal', 'st');

        $use_save_search = $this->session->userdata('auth_type') == 'user';
        $data['use_save_search'] = $use_save_search;

        $data['use_leader'] = $use_leader;

        $this->set_api_content('data', $data);
    }

    /**
     * Add listing to saved list
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function save($listing_id)
    {
        $auth_type = $this->session->userdata('auth_type');
        if ('user' != $auth_type) {
            log_message('error', 'listings API: Wrong auth type ("' . $auth_type . '")');
            $this->set_api_content('errors', l('api_error_wrong_auth_type', 'listings'));
            return false;
        }

        $user_id = $this->session->userdata('user_id');

        $listing_id = intval($listing_id);
        if (!$listing_id) {
            log_message('error', 'listings API: Empty listing id');
            $this->set_api_content('errors', l('api_error_empty_listing_id', 'listings'));
            return false;
        }

        $this->Listings_model->set_format_settings('use_format', false);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);
        if (!$listing || $user_id != $listing['id_user']) {
            log_message('error', 'listings API: Invalid listing id');
            $this->set_api_content('errors', l('api_error_invalid_listing_id', 'listings'));
            return false;
        }

        $this->load->model('listings/models/Saved_listings_model');

        if ($this->session->userdata('auth_type') == 'user') {
            $user_id = $this->session->userdata('user_id');
            $saved_listings = $this->session->userdata('saved_listings');
            if (!isset($saved_listings) || empty($saved_listings)) {
                $this->load->model('listings/models/Saved_listings_model');
                $list = $this->Saved_listings_model->get_saved_list_by_user((int) $user_id);
                $saved_listings = array();
                foreach ($list as $item) {
                    $saved_listings[] = $item['id_listing'];
                }
                $this->session->set_userdata('saved_listings', $saved_listings);
            }
        } elseif (isset($_SESSION['saved_listings'])) {
            $saved_listings = $_SESSION['saved_listings'];
        } else {
            $saved_listings = array();
        }

        $listing_ids = array();
        foreach ($saved_listings as $saved) {
            $listing_ids = $saved['id_listing'];
        }
        if (in_array($listing_id, $listing_ids)) {
            log_message('error', 'listings API: Listing already saved');
            $this->set_api_content('errors', l('api_error_already_saved', 'listings'));
            return false;
        }

        $data['id_user'] = $user_id;
        $data['id_listing'] = $listing_id;
        $this->Saved_listings_model->save_saved(null, $data);

        $this->set_api_content('messages', l('success_listings_saved', 'listings'));
    }

    /**
     * Remove listing from saved list
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function delete_saved($listing_id)
    {
        $auth_type = $this->session->userdata('auth_type');
        if ('user' != $auth_type) {
            log_message('error', 'listings API: Wrong auth type ("' . $auth_type . '")');
            $this->set_api_content('errors', l('api_error_wrong_auth_type', 'listings'));
            return false;
        }

        $user_id = $this->session->userdata('user_id');

        $listing_id = intval($listing_id);
        if (!$listing_id) {
            log_message('error', 'listings API: Empty listing id');
            $this->set_api_content('errors', l('api_error_empty_listing_id', 'listings'));
            return false;
        }

        $this->load->model('listings/models/Saved_listings_model');
        $this->Saved_listings_model->delete_saved($user_id, $listing_id);

        $this->set_api_content('messages', l('success_delete_saved', 'listings'));
    }

    /**
     * Get data for search form
     * 
     * @return void
     */
    public function get_search_data()
    {
        $this->load->model('Properties_model');
        $property_types = $this->Properties_model->get_categories('property_types', $this->pg_language->current_lang_id);
        $property_items = $this->Properties_model->get_all_properties('property_types', $this->pg_language->current_lang_id);

        $data['properties'] = array();

        foreach ($property_types as $property_type_gid => $property_type_value) {
            $data['properties'][] = array('gid' => $property_type_gid, 'name' => $property_type_value, 'level' => 1);
            foreach ($property_items[$property_type_gid] as $property_item_gid => $property_item_value) {
                $data['properties'][] = array('gid' => $property_type_gid . '_' . $property_item_gid, 'name' => $property_item_value, 'level' => 2);
            }
        }

        $this->load->model('Countries_model');
        $countries = $this->Countries_model->get_countries(array('priority' => 'ASC'), array(), array(), $this->pg_language->current_lang_id);
        $data['countries']['count'] = count($countries);
        if ($data['countries']['count']) {
            foreach ($countries as $country) {
                $data['countries']['items'][] = array(
                    'id' => $country['id'],
                    'code' => $country['code'],
                    'name' => $country['name'],
                );
            }
        }

        $this->set_api_content('data', $data);
    }

    /**
     * Get listings edit data
     * 
     * @param string $listing_id listing identifier
     * @return void
     */
    public function get_edit_data($listing_id = null)
    {
        if ($listing_id) {
            $listing = $this->Listings_model->get_listing_by_id($listing_id);
        } else {
            $listing = array();
        }

        $data['types'] = $this->Listings_model->get_operation_types();
        $data['types'] = array_flip($data['types']);
        foreach ($data['types'] as $gid => $id) {
            $data['types'][$gid] = l('operation_type_' . $gid, 'listings');
        }

        $this->load->model('Properties_model');
        $property_types = $this->Properties_model->get_categories('property_types', $this->pg_language->current_lang_id);
        $property_items = $this->Properties_model->get_all_properties('property_types', $this->pg_language->current_lang_id);

        $data['properties'] = array();

        if (!empty($listing)) {
            foreach ($property_items[$listing['id_category']] as $property_item_gid => $property_item_value) {
                $data['properties'][] = array('gid' => $listing['id_category'] . '_' . $property_item_gid, 'name' => $property_item_value);
            }
        } else {
            foreach ($property_types as $property_type_gid => $property_type_value) {
                foreach ($property_items[$property_type_gid] as $property_item_gid => $property_item_value) {
                    $data['properties'][] = array('gid' => $property_type_gid . '_' . $property_item_gid, 'name' => $property_item_value, 'group' => $property_type_value);
                }
            }
        }

        $this->load->model('Countries_model');
        $countries = $this->Countries_model->get_countries(array('priority' => 'ASC'), array(), array(), $this->pg_language->current_lang_id);
        $data['countries']['count'] = count($countries);
        if ($data['countries']['count']) {
            foreach ($countries as $country) {
                $data['countries']['items'][] = array(
                    'id' => $country['id'],
                    'code' => $country['code'],
                    'name' => $country['name'],
                );
            }
        }

        if ($this->pg_module->is_module_installed('currencies')) {
            $this->load->model('Currencies_model');
            $currencies = $this->Currencies_model->get_currency_list();
            $data['currencies'] = $currencies;
        }

        $square_units = ld('square_units', 'listings');
        $data['units'] = $square_units['option'];

        $price_periods = ld('price_period', 'listings');
        $data['price_periods'] = $price_periods['option'];

        $price_types = ld('price_type', 'listings');
        $data['price_types'] = $price_types['option'];

        $this->set_api_content('data', $data);
    }

    /**
     * Get data of user listings
     * 
     * @param integer $user_id user identifier
     * @return void
     */
    public function user($user_id)
    {
        $filters['active'] = '1';
        $filters['id_user'] = $user_id;

        $operation_types = $this->Listings_model->get_operation_types();

        $type = $this->input->post('type');
        if (in_array($type, $operation_types))
            $filters['type'] = $type;

        $order = $this->input->post('order');
        $data['order'] = trim(strip_tags($order));
        if (!$data['order'])
            $data['order'] = 'date_created';

        $order_direction = $this->input->post('order_direction');
        $data['order_direction'] = strtoupper(trim(strip_tags($order_direction)));
        if ($data['order_direction'] != 'DESC')
            $data['order_direction'] = 'ASC';

        $page = $this->input->post('page');
        $data['page'] = $page;
        if (!$data['page'])
            $data['page'] = 1;

        $items_on_page = $this->pg_module->get_module_config('start', 'index_items_per_page');

        $data['listings_count'] = $this->Listings_model->get_listings_count($filters);

        $use_leader = false;
        if ($data['listings_count'] > 0) {
            $order_array = array();

            if ($order == 'default') {
                if (isset($data['id_city']) && !empty($data['id_city']))
                    $order_array['is_lift_up_city'] = 'DESC';
                if (isset($data['id_region']) && !empty($data['id_region']))
                    $order_array['is_lift_up_region'] = 'DESC';
                if (isset($data['id_country']) && !empty($data['id_country']))
                    $order_array['is_lift_up_country'] = 'DESC';
                $order_array['is_lift_up'] = 'DESC';
                $order_array['id'] = 'DESC';
                $use_leader = true;
            }else {
                $order_array[$data['order']] = $data['order_direction'];
            }

            $this->Listings_model->set_format_settings('get_user', false);
            $listings = $this->Listings_model->get_listings_list($filters, $data['page'], $items_on_page, $order_array);
            $this->Listings_model->set_format_settings('get_user', true);

            foreach ($listings as $i => $listing) {
                $data['listings'][$i] = array(
                    'listing_id' => $listing['id'],
                    'logo_image' => $listing['media']['photo']['thumbs'],
                );

                unset($listing['id']);
                unset($listing['logo_image']);
                unset($listing['media']);
                unset($listing['listing_file']);
                unset($listing['listing_file_date']);
                unset($listing['listing_video']);
                unset($listing['listing_video_image']);
                unset($listing['listing_video_data']);

                foreach ($listing as $name => $value) {
                    $data['listings'][$i]['listing_' . $name] = $value;
                }
            }
        } else {
            $this->set_api_content('messages', l('api_error_listing_not_found', 'listings'));
        }

        $data['date_format'] = $this->pg_date->get_format('date_literal', 'st');
        $data['date_time_format'] = $this->pg_date->get_format('date_time_literal', 'st');

        $use_save_search = $this->session->userdata('auth_type') == 'user';
        $data['use_save_search'] = $use_save_search;

        $data['use_leader'] = $use_leader;

        $this->set_api_content('data', $data);
    }

    /**
     * Share listing by mail
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function share_by_mail($listing_id)
    {
        $post_data = array(
            'email' => $this->input->post('email', true),
            'user' => $this->input->post('username', true),
            'message' => $this->input->post('comment', true),
        );

        $validate_data = $this->Listings_model->validate_share($listing_id, $post_data);

        /* $is_captcha_valid = ($this->input->post('code', true) == $this->session->userdata('captcha_word')) ? 1 : 0;
          if(!$is_captcha_valid){
          $validate_data['errors'][] = l('error_invalid_captcha', 'contact');
          } */

        if (!empty($validate_data['errors'])) {
            $this->set_api_content('error', $validate_data['errors']);
        } else {
            $email = $validate_data['data']['email'];
            unset($validate_data['data']['email']);

            $this->load->helper('seo');
            $this->load->library('Translit');

            $current_lang = $this->pg_language->get_lang_by_id($this->pg_language->current_lang_id);
            $current_lang_code = $current_lang['code'];

            //get listing
            $listing = $this->Listings_model->get_listing_by_id($listing_id);

            $validate_data['data']['listing_id'] = $listing_id;
            $validate_data['data']['listing_name'] = $listing['output_name'];
            $validate_data['data']['listing_owner'] = $listing['user']['output_name'];
            $validate_data['data']['listing_url'] = rewrite_link('listings', 'view', $listing);

            //send mail
            $this->load->model('Notifications_model');
            $this->Notifications_model->send_notification($email, 'share_listing', $validate_data['data'], '', $this->pg_language->current_lang_id);

            $this->set_api_content('success', l('success_share_send', 'listings'));
        }
    }

    /**
     * Get field editor structure
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function get_field_editor($listing_id)
    {
        $this->Listings_model->set_format_settings('get_description', true);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('get_description', false);

        $this->load->model('Field_editor_model');
        $this->Field_editor_model->initialize($listing['field_editor_type']);
        $property_type_sections = $this->Field_editor_model->get_section_list();
        $sections = array();
        foreach ($property_type_sections as $section) {
            $sections[] = $section['gid'];
        }
        $data['sections_data'] = array_values($property_type_sections);

        if (!empty($sections)) {
            $params['where']['editor_type_gid'] = $listing['field_editor_type'];
            $params['where_in']['section_gid'] = $sections;
            $fields_data = $this->Field_editor_model->get_form_fields_list($listing, $params);
            $data['fields_data'] = array_values($fields_data);
        }

        $this->set_api_content('data', $data);
    }

    /**
     * Activate listing by identifier
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function activate($listing_id)
    {
        $user_id = $this->session->userdata('user_id');
        $user_type = $this->session->userdata('user_type');

        $this->Listings_model->set_format_settings(array('get_user' => false, 'get_location' => false));
        $listing_data = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings(array('get_user' => true, 'get_location' => true));
        if ($user_id != $listing_data['id_user']) {
            show_404();
            return;
        }

        if ($this->pg_module->is_module_installed('users_services')) {
            ///// check pay services exists
            $services_params = array(
                'where_in' => array(
                    'template_gid' => array($user_type . '_post_template', $user_type . '_combined_template'),
                ),
                'where' => array(
                    'status' => 1,
                ),
            );
            $this->load->model('Services_model');
            $service_count = $this->Services_model->get_service_count($services_params);
            if ($service_count > 0) {
                $data['use_services'] = true;
            }
        } else {
            $data['use_services'] = false;
        }

        if ($data['use_services']) {
            $this->load->model('Users_services_model');
            $user_service_id = intval($this->input->post('id_user_service', true));
            $user_service = $this->Users_services_model->get_service($user_service_id);
            if (empty($user_service) || $user_service['id_user'] != $user_id || !$user_service['post_count'] || !$user_service['status']) {
                $this->set_api_content('error', l('error_service_activating', 'listings'));
                exit;
            } else {
                $post_period = $user_service['post_period'];
                $user_service['post_count'] --;
                if ($user_service['post_count'] <= 0) {
                    $user_service['post_count'] = $user_service['status'] = 0;
                }
                $this->Users_services_model->save_service($user_service_id, $user_service);
            }
        } else {
            $post_period = $this->pg_module->get_module_config('listings', 'default_activation_period');
        }

        if (!$listing_data['initial_moderation']) {
            $activate_data['initial_activity'] = $post_period;
            $this->Listings_model->save_listing($listing_id, $activate_data, false);
        } else {
            if (strtotime($listing_data['date_expire']) > time()) {
                $date_expire = date('Y-m-d H:i:s', strtotime($listing_data['date_expire']) + $post_period * 24 * 60 * 60);
            } else {
                $date_expire = date('Y-m-d H:i:s', time() + $post_period * 24 * 60 * 60);
            }
            $this->Listings_model->activate_listing($listing_id, 1, $date_expire);
        }

        $this->set_api_content('success', l('success_service_activating', 'listings'));
    }

    /**
     * Deactivate listing by identifier
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function deactivate($listing_id)
    {
        $user_id = $this->session->userdata('user_id');
        $this->Listings_model->set_format_settings(array('get_user' => false, 'get_location' => false));
        $listing_data = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings(array('get_user' => true, 'get_location' => true));
        if ($user_id != $listing_data['id_user']) {
            show_404();
            return;
        }
        if ($listing_data['status'] == 1)
            $this->Listings_model->activate_listing($listing_id, 0);
        $this->set_api_content('success', l('success_service_deactivating', 'listings'));
    }

    /**
     * Reactivate listing by identifier
     * 
     * @param ineteger $listing_id listing identifier
     * @return void
     */
    public function reactivate($listing_id)
    {
        $user_id = $this->session->userdata('user_id');
        $this->Listings_model->set_format_settings(array('get_user' => false, 'get_location' => false));
        $listing_data = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings(array('get_user' => true, 'get_location' => true));
        if ($user_id != $listing_data['id_user']) {
            show_404();
            return;
        }
        if ($listing_data['status'] == 0 && strtotime($listing_data['date_expire']) > time()) {
            $this->Listings_model->activate_listing($listing_id, 1);
        }
        $this->set_api_content('success', l('success_service_activating', 'listings'));
    }

    /**
     * Mark listing as sold by identifier
     * 
     * @param ineteger $listing_id listing identifier
     * @return void
     */
    public function make_sold($listing_id)
    {
        $user_id = $this->session->userdata('user_id');
        $this->Listings_model->set_format_settings(array('get_user' => false, 'get_location' => false));
        $listing_data = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings(array('get_user' => true, 'get_location' => true));
        if ($user_id != $listing_data['id_user']) {
            show_404();
            return;
        }
        $this->Listings_model->make_sold($listing_id, 1);
        $this->set_api_content('success', l('success_make_sold', 'listings'));
    }

    /**
     * Remove sold status from listing by identifier
     * 
     * @param ineteger $listing_id listing identifier
     * @return void
     */
    public function delete_sold($listing_id)
    {
        $user_id = $this->session->userdata('user_id');
        $this->Listings_model->set_format_settings(array('get_user' => false, 'get_location' => false));
        $listing_data = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings(array('get_user' => true, 'get_location' => true));
        if ($user_id != $listing_data['id_user']) {
            show_404();
            return;
        }
        $this->Listings_model->make_sold($listing_id, 0);
        $this->set_api_content('success', l('success_remove_sold', 'listings'));
    }

    /**
     * Get listings services
     * 
     * @return void
     */
    public function services()
    {
        $user_id = $this->session->userdata('user_id');

        if ($this->pg_module->is_module_installed('services')) {
            $this->load->model('Services_model');
            ///// check pay services exists
            $templates_gids = array(
                'listings_featured_template',
                'listings_lift_up_template',
                'listings_lift_up_country_template',
                'listings_lift_up_region_template',
                'listings_lift_up_city_template',
                'listings_highlight_template',
                'listings_slide_show_template',
            );
            $services = $this->Services_model->get_active_services($user_id, $templates_gids);
            $data['services'] = $services;
        } else {
            $data['services'] = array();
        }

        $this->set_api_content('data', $data);
    }

    /**
     * Apply service to listing
     * 
     * @param integer $listing_id listing identifier
     * @param integer $service_id service identifier
     * @return void
     */
    public function apply_service($listing_id, $service_id)
    {
        $user_id = $this->session->userdata('user_id');

        $listing_data = $this->Listings_model->get_listing_by_id($listing_id);
        if (empty($listing_data) || $user_id != $listing_data['id_user']) {
            show_404();
            return;
        }

        $this->load->helper('payments');
        $params = array('listing' => $listing_id);
        post_location_request(site_url() . 'services/form/' . $service_gid, $params);
    }

    /**
     * Save listing photo data
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function set_photo_data($listing_id)
    {
        $photo_id = intval($this->input->post('photo_id', true));
        if (!$photo_id) {
            $this->set_api_content('error', l('error_not_access', 'listings'));
            exit;
        }

        $this->load->model('Upload_gallery_model');

        $photo = $this->Upload_gallery_model->get_file_by_id($photo_id);
        if ($photo['object_id'] != $listing_id) {
            $this->set_api_content('error', l('error_not_access', 'listings'));
            exit;
        }

        $user_id = $this->session->userdata('user_id');
        $this->Listings_model->set_format_settings('use_format', false);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);
        if ($user_id != $listing['id_user']) {
            $this->set_api_content('error', l('error_not_access', 'listings'));
            exit;
        }

        $this->load->model('listings/models/Listings_uploads_model');
        $photo_type = $this->Listings_uploads_model->get_gallery_type();

        $post_data['comment'] = $this->input->post('comment', true);
        $validate_data = $this->Upload_gallery_model->validate_file_data($photo_id, $photo_type['id'], $post_data);
        if (!empty($validate_data['errors'])) {
            $return['errors'] = $validate_data['errors'];
        } else {
            $data = $validate_data['data'];
            $data['type_id'] = $photo_type['id'];
            $data['object_id'] = $photo['object_id'];
            $return = $this->Upload_gallery_model->save_file($photo_id, $data, '', true);
        }

        if (!empty($return['errors'])) {
            $this->set_api_content('error', implode('<br>', $return['errors']));
        } else {
            $this->set_api_content('messages', l('photo_successfully_saved', 'listings'));
        }

        unset($return['errors']);

        $this->set_api_content('data', $return);
    }

    /**
     * Set map coordinates
     * 
     * @param integer $listing_id listing identifier
     * @return void
     */
    public function set_map_coordinates($listing_id)
    {
        $user_id = $this->session->userdata('user_id');

        $this->Listings_model->set_format_settings('use_format', false);
        $listing = $this->Listings_model->get_listing_by_id($listing_id);
        $this->Listings_model->set_format_settings('use_format', true);

        if (empty($listing) || $listing['id_user'] != $user_id) {
            $this->set_api_content('error', l('error_not_access', 'listings'));
            exit;
        }

        $this->load->model('Geomap_model');
        $this->load->model('geomap/models/Geomap_settings_model');

        $map_gid = $this->Geomap_model->get_default_driver_gid();
        $settings = $this->Geomap_settings_model->get_settings($map_gid, $user_id, $listing_id, 'listing_view');
        if (!$settings)
            $settings = array('gid' => 'listing_view');
        $settings['lat'] = $save_data['lat'] = $this->input->post('lat', true);
        $settings['lon'] = $save_data['lon'] = $this->input->post('lon', true);

        $validate_data = $this->Geomap_settings_model->validate_settings($settings);
        if (empty($validate_data['errors']) && !empty($validate_data['data'])) {
            $this->Geomap_settings_model->save_settings($map_gid, 0, $listing_id, 'listing_view', $validate_data['data']);
        }

        $validate_data = $this->Listings_model->validate_listing($listing_id, $save_data);
        if (empty($validate_data['errors']) && !empty($validate_data['data'])) {
            $this->Listings_model->save_listing($listing_id, $validate_data['data']);
        }

        $this->set_api_content('success', l('success_listings_saved', 'listings'));
    }

}
