<?php

namespace Pg\Modules\Listings\Models;

/**
 * Listings module
 *
 * @package 	PG_RealEstate
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */

/**
 * Listings Module Model
 * 
 * @package 	PG_RealEstate
 * @subpackage 	Listings
 * @category	models
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Listings_module_model extends \Model
{

    const OPERATION_SALE = 'sale';
    const OPERATION_BUY = 'buy';
    const OPERATION_RENT = 'rent';
    const OPERATION_LEASE = 'lease';
    const OPERATION_SOLD = 'sold';
    const OPERATION_ALL = 'all';

    /**
     * Link to CodeIgniter object
     * 
     * @var object
     */
    protected $ci;

    /**
     * Period activity of listings services in days by default
     * 
     * @var integer
     */
    public $service_period_default = 14;

    /**
     * Steps number in price range
     * 
     * @var integer
     */
    public $price_range_steps_default = 50;

    /**
     * Steps number in square range
     * 
     * @var integer
     */
    public $square_range_steps_default = 50;

    /**
     * Max price by default
     * 
     * @var integer
     */
    public $price_max_default = 100000;

    /**
     * Max square by default
     * 
     * @var integer
     */
    public $square_max_default = 100;

    /**
     * Constructor
     *
     * return Listings_module_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
        $this->ci->load->model('listings/models/Listings_model');
    }

    /**
     * Update statistics on save listing
     * 
     * @param array $listing_old old listing data
     * @param array $listing_new new listing data
     * @return void
     */
    public function onsave_listing($listing_old, $listing_new)
    {
        $this->_update_price_range($listing_old, $listing_new);
        $this->_update_square_range($listing_old, $listing_new);
    }

    /**
     * Update statistics on remove listing
     * 
     * @param array $listing listing data
     * @return void
     */
    public function ondelete_listing($listing)
    {
        $this->_update_price_range($listing);
        $this->_update_square_range($listing);
    }

    /**
     * Update listing price range
     * 
     * @param array $listing_old old listing data
     * @param array $listing_new new listing data
     * @return void
     */
    public function _update_price_range($listing_old, $listing_new = null)
    {
        if (empty($listing_old['status']) && empty($listing_new['status'])) {
            return;
        }
        
        $this->ci->load->model('Listings_model');
        
        $operation_type_old = $this->ci->Listings_model->getOperationTypeById($listing_old['id_type']);
        if ($listing_old[self::OPERATION_SOLD] == 1) {
            $operation_type_old = self::OPERATION_SOLD;
        }

        if (!empty($listing_new['id_type'])){
            $operation_type_new = $this->ci->Listings_model->getOperationTypeById($listing_new['id_type']);
            if ($listing_new[self::OPERATION_SOLD] == 1) {
                $operation_type_new = self::OPERATION_SOLD;
            }
        } else {
            $operation_type_new = null;
        }

        $price_range = $this->ci->Listings_model->getPriceRange(-1, $operation_type_old);
        $this->ci->pg_module->set_module_config('listings', 'price_min_' . $operation_type_old, (float) $price_range['min']);
        $this->ci->pg_module->set_module_config('listings', 'price_max_' . $operation_type_old, (float) $price_range['max']);

        if (isset($operation_type_new) && $operation_type_old != $operation_type_new){
            $price_range = $this->ci->Listings_model->getPriceRange(-1, $operation_type_new);
            $this->ci->pg_module->set_module_config('listings', 'price_min_' . $operation_type_new, (float) $price_range['min']);
            $this->ci->pg_module->set_module_config('listings', 'price_max_' . $operation_type_new, (float) $price_range['max']);
        }

    }

    /**
     * Update listing square range
     * 
     * @param array $listing_old old listing data
     * @param array $listing_new new listing data
     * @return void
     */
    public function _update_square_range($listing_old, $listing_new = null)
    {
        if (empty($listing_old['status']) && empty($listing_new['status']))
            return;

        if (empty($listing_new)) {
            $listing_new['status'] = 0;
        } elseif (!isset($listing_new['status'])) {
            $listing_new['status'] = $listing_old['status'];
        }

        $square_min = 0;
        $square_max = 0;

        $operation_type_new = null;
        $operation_type_old = null;

        if (!empty($listing_old)) {
            $operation_type_old = $this->ci->Listings_model->getOperationTypeById($listing_old['id_type']);
            if (!$operation_type_new) {
                $operation_type_new = $operation_type_old;
            }
            $square_min = $listing_old['square'];
            $square_max = (($operation_type_old == 'buy' || $operation_type_old == 'lease') && !empty($listing_old['square_max'])) ? $listing_old['square_max'] : $listing_old['square'];

            if ($listing_old['sold']) {
                $operation_type_old = 'sold';
            }

            if (!isset($listing_new['square'])) {
                $listing_new['square'] = $listing_old['square'];
            }
            if (!isset($listing_new['square_max'])) {
                $listing_new['square_max'] = $listing_old['square_max'];
            }
        }

        if (isset($listing_new['id_type'])) {
            $type_new_id = $listing_new['id_type'];
        } elseif ($listing_old['id_type']) {
            $type_new_id = $listing_old['id_type'];
        }

        if (isset($listing_new['sold'])) {
            $sold_new = $listing_new['sold'];
        } elseif ($listing_old['sold']) {
            $sold_new = $listing_old['sold'];
        } else {
            $sold_new = 0;
        }

        if ($sold_new) {
            $operation_type_new = 'sold';
        } elseif ($type_new_id) {
            $operation_types = $this->ci->Listings_model->get_operation_types();
            $operation_type_new = $operation_types[$type_new_id];
        } else {
            $operation_type_new = $operation_type_old;
        }

        if (empty($listing_old['status'])) {
            $operation_type_old = null;
        }

        if ($operation_type_old) {
            $min_square_old = $this->ci->pg_module->get_module_config('listings', 'square_min_' . $operation_type_old);
            $max_square_old = $this->ci->pg_module->get_module_config('listings', 'square_max_' . $operation_type_old);
            if ($square_min > 0 && $square_min == $min_square_old || $square_max > $this->square_max_default && $square_max == $max_square_old) {
                $square_range = $this->ci->Listings_model->getSquareRange($listing_old['id'], $operation_type_old);
                $square_min_old = $square_range['min'];
                $square_max_old = max($square_range['max'], $this->square_max_default);
                $this->ci->pg_module->set_module_config('listings', 'square_min_' . $operation_type_old, (float) $square_min_old);
                $this->ci->pg_module->set_module_config('listings', 'square_max_' . $operation_type_old, (float) $square_max_old);
                if ($operation_type_new == $operation_type_old) {
                    $square_min = $square_min_old;
                    $square_max = $square_max_old;
                }
            }
        }

        if (empty($listing_new['status'])) {
            $operation_type_new = null;
        }

        if ($operation_type_new) {
            $min_square_new = $this->ci->pg_module->get_module_config('listings', 'square_min_' . $operation_type_new);
            $max_square_new = $this->ci->pg_module->get_module_config('listings', 'square_max_' . $operation_type_new);

            if ($operation_type_new == 'buy' || $operation_type_new == 'lease') {
                if (isset($listing_new['square_max'])) {
                    $square_max = $listing_new['square_max'];
                }
                if (isset($listing_new['square'])) {
                    $square_min = $listing_new['square'];
                }
            } else {
                if (isset($listing_new['square'])) {
                    $square_max = $listing_new['square'];
                    $square_min = $listing_new['square'];
                }
            }

            if ($square_min > 0 && (!$min_square_new || $square_min < $min_square_new)) {
                $this->ci->pg_module->set_module_config('listings', 'square_min_' . $operation_type_new, (float) $square_min);
            }
            if ($square_max > 0 && ($square_max > $max_square_new)) {
                $square_max = max($square_max, $this->square_max_default);
                $this->ci->pg_module->set_module_config('listings', 'square_max_' . $operation_type_new, (float) $square_max);
            }
        }
    }

    /**
     * Return price range as associated array by operation type
     * 
     * @param string $type operation type
     * @return array
     */
    public function get_price_range($type = null)
    {
        $price_range = array();
        $operation_types = $this->ci->Listings_model->get_operation_types(true);
        if ($type && in_array($type, $operation_types)) {
            $operation_types = array($type);
        }
        foreach ($operation_types as $type) {
            if ($type == 'all') {
                $min_price = 0;
                $max_price = 0;
                foreach ($this->ci->Listings_model->operations_arr as $v) {
                    $min_p = (float) $this->ci->pg_module->get_module_config('listings', 'price_min_' . $v);
                    $max_p = (float) $this->ci->pg_module->get_module_config('listings', 'price_max_' . $v);

                    if ($min_price) {
                        $min_price = min($min_price, $min_p);
                    } else {
                        $min_price = $min_p;
                    }

                    if ($max_price) {
                        $max_price = max($max_price, $max_p);
                    } else {
                        $max_price = $max_p;
                    }
                }
            } else {
                $min_price = (float) $this->ci->pg_module->get_module_config('listings', 'price_min_' . $type);
                $max_price = (float) $this->ci->pg_module->get_module_config('listings', 'price_max_' . $type);
            }

            $this->load->helper('start');

            $base_currency_gid = $this->ci->pg_module->get_module_config("start", "base_currency");

            $min_price = get_currency_value(array('value' => $min_price, 'cur_gid' => $base_currency_gid));
            $max_price = get_currency_value(array('value' => $max_price, 'cur_gid' => $base_currency_gid));

            $step_val = max(ceil(($max_price - $min_price) / $this->price_range_steps_default), 1);
            $range = array();
            for ($i = $min_price; $i < $max_price; $i+=$step_val) {
                $range[] = $i;
            }
            $range[] = $max_price;
            $price_range[$type] = array(
                'min_price' => $min_price,
                'max_price' => $max_price,
                'data' => $range,
            );
        }
        return $price_range;
    }

    /**
     * Return square range by operation type
     * 
     * @param string $type operation type
     * @return array
     */
    public function get_square_range($type = null)
    {
        $square_range = array();
        $operation_types = $this->ci->Listings_model->get_operation_types(true);
        if ($type && in_array($type, $operation_types)) {
            $operation_types = array($type);
        }
        foreach ($operation_types as $type) {
            if ($type == 'all') {
                $min_square = 0;
                $max_square = 0;
                foreach ($this->ci->Listings_model->operations_arr as $v) {
                    $min_p = (float) $this->ci->pg_module->get_module_config('listings', 'square_min_' . $v);
                    $max_p = (float) $this->ci->pg_module->get_module_config('listings', 'square_max_' . $v);

                    if ($min_square) {
                        $min_square = min($min_square, $min_p);
                    } else {
                        $min_square = $min_p;
                    }

                    if ($max_square) {
                        $max_square = max($max_square, $max_p);
                    } else {
                        $max_square = $max_p;
                    }
                }
            } else {
                $min_square = (float) $this->ci->pg_module->get_module_config('listings', 'square_min_' . $type);
                $max_square = (float) $this->ci->pg_module->get_module_config('listings', 'square_max_' . $type);
            }

            $step_val = max(ceil(($max_square - $min_square) / $this->square_range_steps_default), 1);
            $range = array();
            if ($min_square) {
                $range[] = $min_square;
            }
            for ($i = $min_square; $i < $max_square; $i+=$step_val) {
                $range[] = $i;
            }
            $range[] = $max_square;
            $square_range[$type] = array(
                'min_square' => $min_square,
                'max_square' => $max_square,
                'data' => $range,
            );
        }

        return $square_range;
    }

}
