<?php

namespace Pg\Modules\Site_map\Models;

/**
 * Site map module
 *
 * @package 	PG_RealEstate
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
if (!defined('BASEPATH')) {
    exit('No direct script access allowed');
}
define('SITEMAP_MODULES_TABLE', DB_PREFIX . 'sitemap_modules');

/**
 * Site map main model
 *
 * @package 	PG_RealEstate
 * @subpackage 	Site map
 *
 * @category	models
 *
 * @copyright 	Copyright (c) 2000-2014 PG Real Estate - php real estate listing software
 * @author 		Pilot Group Ltd <http://www.pilotgroup.net/>
 */
class Site_map_model extends \Model
{

    /**
     * Link to CodeIgniter object
     *
     * @var object
     */
    protected $ci;

    /**
     * Pages cache
     *
     * @var array
     */
    protected $sitemap_module_cache = array();

    /**
     * Constructor
     *
     * @return Site_map_model
     */
    public function __construct()
    {
        parent::__construct();
        $this->ci = &get_instance();
    }

    /**
     * Return site map of module by GUID
     *
     * @param string $module_gid module GUID
     *
     * @return array/false
     */
    public function get_sitemap_module_by_gid($module_gid)
    {
        if (empty($this->sitemap_module_cache[$module_gid])) {
            $this->ci->db->select('id, module_gid, model_name, get_urls_method')->from(SITEMAP_MODULES_TABLE)->where("module_gid",
                                                                                                                     $module_gid);
            $results = $this->ci->db->get()->result_array();
            if (!empty($results)) {
                $this->sitemap_module_cache[$module_gid] = $results[0];
            }
        }

        return (!empty($this->sitemap_module_cache[$module_gid])) ? $this->sitemap_module_cache[$module_gid] : false;
    }

    /**
     * Return site maps by modules as array
     *
     * @return array
     */
    public function get_sitemap_modules()
    {
        $this->ci->db->select('id, module_gid, model_name, get_urls_method')->from(SITEMAP_MODULES_TABLE);
        $results = $this->ci->db->get()->result_array();
        if (!empty($results)) {
            foreach ($results as $r) {
                $this->sitemap_module_cache[$r["module_gid"]] = $r;
            }
        }

        return $this->sitemap_module_cache;
    }

    /**
     * Add module to site map
     *
     * @param string $module_gid module GUID
     * @param array  $data       module data
     *
     * @return void
     */
    public function set_sitemap_module($module_gid, $data = array())
    {
        $module_data = $this->get_sitemap_module_by_gid($module_gid);
        if (empty($module_data)) {
            $this->ci->db->insert(SITEMAP_MODULES_TABLE, $data);
        } else {
            $this->ci->db->where("module_gid", $module_gid);
            $this->ci->db->update(SITEMAP_MODULES_TABLE, $data);
        }
    }

    /**
     * Remove module from site map
     *
     * @param string $module_gid module GUID
     *
     * @return void
     */
    public function delete_sitemap_module($module_gid)
    {
        $this->ci->db->where("module_gid", $module_gid);
        $this->ci->db->delete(SITEMAP_MODULES_TABLE);
    }

    /**
     * Return site map data
     *
     * @return array
     */
    public function get_sitemap_links()
    {
        $modules = $this->get_sitemap_modules();
        $blocks = array();

        foreach ($modules as $module) {
            $this->ci->load->model($module["module_gid"] . "/models/" . $module["model_name"]);
            $blocks[] = $this->ci->{$module["model_name"]}->{$module["get_urls_method"]}();
        }

        return $blocks;
    }

    // seo

    /**
     * Return data for rewriting of seo urls
     *
     * @param string  $method  method name
     * @param integer $lang_id language identifier
     *
     * @return array
     */
    public function get_seo_settings($method = '', $lang_id = '')
    {
        if (!empty($method)) {
            return $this->_get_seo_settings($method, $lang_id);
        } else {
            $actions = array('index');
            $return = array();
            foreach ($actions as $action) {
                $return[$action] = $this->_get_seo_settings($action, $lang_id);
            }

            return $return;
        }
    }

    /**
     * Return data for rewriting of seo urls (internal)
     *
     * @param string  $method  method name
     * @param integer $lang_id language identifier
     *
     * @return array
     */
    public function _get_seo_settings($method, $lang_id = '')
    {
        if ($method == "index") {
            return array(
                "templates" => array(),
                "url_vars" => array(),
            );
        }
    }

    /**
     * Transform seo value of query string to module variables
     *
     * @param string $var_name_from variable form url
     * @param string $var_name_to   method variable
     * @param string $value         variable value
     *
     * @return mixed
     */
    public function request_seo_rewrite($var_name_from, $var_name_to, $value)
    {
        return $value;
    }

    /**
     * Return data for generating xml sitemap
     *
     * @return array
     */
    public function get_sitemap_xml_urls()
    {
        $this->ci->load->helper('seo');

        $lang_canonical = true;

        if ($this->ci->pg_module->is_module_active('seo')) {
            $lang_canonical = $this->ci->pg_module->get_module_config('seo', 'lang_canonical');
        }

        if ($lang_canonical) {
            $default_lang_id = $this->ci->pg_language->get_default_lang_id();
            $default_lang_code = $this->ci->pg_language->get_lang_code_by_id($default_lang_id);
            $langs[$default_lang_id] = $default_lang_code;
        } else {
            foreach ($this->ci->pg_language->languages as $lang_id => $lang_data) {
                $langs[$lang_id] = $lang_data['code'];
            }
        }

        $return = array();

        $user_settings = $this->pg_seo->get_settings('user', 'site_map', 'index');
        if (!$user_settings['noindex']) {
            foreach ($langs as $lang_id => $lang_code) {
                $return[] = array(
                    "url" => rewrite_link('site_map', 'index', array(), false, $lang_code, $lang_canonical),
                    "priority" => 0.6,
                );
            }
        }

        return $return;
    }

    // banners callback method

    /**
     * Return available pages of banner placements
     *
     * Callback method for banners module
     *
     * @return array
     */
    public function _banner_available_pages()
    {
        $return[] = array("link" => "site_map/index", "name" => l('seo_tags_index_header', 'site_map'));
        return $return;
    }

}
